/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/alignment.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *XAlign	= "GtkAlignment::xalign";
static gchar *YAlign	= "GtkAlignment::yalign";
static gchar *XScale	= "GtkAlignment::xscale";
static gchar *YScale	= "GtkAlignment::yscale";


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkAlignment, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_alignment_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_alignment_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_float_range(XAlign, "X Align:",
			   "The horizontal alignment of the child widget",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(YAlign, "Y Align:",
			   "The vertical alignment of the child widget",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(XScale, "X Scale:",
			   "The horizontal scale of the child widget",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(YScale, "Y Scale:",
			   "The vertical scale of the child widget",
			   0, 1, 0.01, 1, 0, 2);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_alignment_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  property_set_float(XAlign, GTK_ALIGNMENT(widget)->xalign);
  property_set_float(YAlign, GTK_ALIGNMENT(widget)->yalign);
  property_set_float(XScale, GTK_ALIGNMENT(widget)->xscale);
  property_set_float(YScale, GTK_ALIGNMENT(widget)->yscale);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_alignment_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gfloat xalign, yalign, xscale, yscale;
  gboolean myApply;

  xalign = property_get_float(XAlign);
  myApply = Apply;
  yalign = property_get_float(YAlign);
  myApply |= Apply;
  xscale = property_get_float(XScale);
  myApply |= Apply;
  yscale = property_get_float(YScale);
  if (Apply || myApply) gtk_alignment_set(GTK_ALIGNMENT(widget),
					  xalign, yalign, xscale, yscale);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkAlignment, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_alignment_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkAlignment. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_alignment_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkAlignment.
 * Note: as with load(), most work will already have been done.
 */
void
gb_alignment_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  save_float(data, XAlign, GTK_ALIGNMENT(widget)->xalign);
  save_float(data, YAlign, GTK_ALIGNMENT(widget)->yalign);
  save_float(data, XScale, GTK_ALIGNMENT(widget)->xscale);
  save_float(data, YScale, GTK_ALIGNMENT(widget)->yscale);
}



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_alignment_init()
{
  /* Initialise the GTK type */
  gtk_alignment_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = alignment_xpm;
  gbwidget.tooltip = "Alignment";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_alignment_new;
  gbwidget.gb_widget_create_properties	= gb_alignment_create_properties;
  gbwidget.gb_widget_show_properties	= gb_alignment_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_alignment_apply_properties;
  gbwidget.gb_widget_save		= gb_alignment_save;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_alignment_create_popup_menu;
  gbwidget.gb_widget_load		= gb_alignment_load;
  gbwidget.gb_widget_write_source	= gb_alignment_write_source;
*/

  return &gbwidget;
}

