/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/arrow.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Direction	= "GtkArrow::direction";
static gchar *Shadow	= "GtkArrow::shadow";
static gchar *XAlign	= "GtkArrow::xalign";
static gchar *YAlign	= "GtkArrow::yalign";
static gchar *XPad	= "GtkArrow::xpad";
static gchar *YPad	= "GtkArrow::ypad";

static gchar* GbDirectionChoices[] = { "Up", "Down", "Left", "Right", NULL};
static gint GbDirectionValues[] = {
  GTK_ARROW_UP,
  GTK_ARROW_DOWN,
  GTK_ARROW_LEFT,
  GTK_ARROW_RIGHT
};

/* Only 'In' & 'Out' work with Arrows */
static gchar* GbShadowChoices[] = { /*"None", */ "In", "Out",
				    /*"Etched In", "Etched Out", */ NULL};
static gint GbShadowValues[] = {
  /*  GTK_SHADOW_NONE,*/
  GTK_SHADOW_IN,
  GTK_SHADOW_OUT,
  /*GTK_SHADOW_ETCHED_IN,*/
  /*GTK_SHADOW_ETCHED_OUT*/
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkArrow, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
/*
GtkWidget*
gb_arrow_new(GbWidgetNewData *data)
{

}
*/



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_arrow_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Direction, "Direction:", "The direction of the arrow",
		      GbDirectionChoices);
  property_add_choice(Shadow, "Shadow:", "The shadow type of the arrow",
		      GbShadowChoices);
  property_add_float_range(XAlign, "X Align:",
			   "The horizontal alignment of the arrow",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(YAlign, "Y Align:",
			   "The vertical alignment of the arrow",
			   0, 1, 0.01, 1, 0, 2);
  property_add_int_range(XPad, "X Pad:", "The horizontal padding",
			 0, 1000, 1, 10, 1);
  property_add_int_range(YPad, "Y Pad:", "The vertical padding",
			 0, 1000, 1, 10, 1);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_arrow_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbDirectionValues) / sizeof(GbDirectionValues[0]);
       i++) {
    if (GbDirectionValues[i] == GTK_ARROW(widget)->arrow_type)
      property_set_choice(Direction, i);
  }

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_ARROW(widget)->shadow_type)
      property_set_choice(Shadow, i);
  }
  property_set_float(XAlign, GTK_MISC(widget)->xalign);
  property_set_float(YAlign, GTK_MISC(widget)->yalign);
  property_set_int(XPad, GTK_MISC(widget)->xpad);
  property_set_int(YPad, GTK_MISC(widget)->ypad);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_arrow_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *direction, *shadow;
  gboolean myApply;
  gfloat xalign, yalign;
  gint xpad, ypad, i;
  GtkArrowType arrow_type = GTK_ARROW_UP;
  GtkShadowType shadow_type = GTK_SHADOW_IN;

  direction = property_get_choice(Direction);
  myApply = Apply;
  for (i = 0; i < sizeof(GbDirectionValues) / sizeof(GbDirectionValues[0]);
       i++) {
    if (!strcmp(direction, GbDirectionChoices[i])) {
      arrow_type = GbDirectionValues[i];
      break;
    }
  }

  shadow = property_get_choice(Shadow);
  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (!strcmp(shadow, GbShadowChoices[i])) {
      shadow_type = GbShadowValues[i];
      break;
    }
  }

  if (Apply || myApply) {
    gtk_arrow_set(GTK_ARROW(widget), arrow_type, shadow_type);
  }

  xalign = property_get_float(XAlign);
  myApply = Apply; /* save Apply flag */
  yalign = property_get_float(YAlign);
  if (Apply || myApply)
    gtk_misc_set_alignment(GTK_MISC(widget), xalign, yalign);

  xpad = property_get_int(XPad);
  myApply = Apply; /* save Apply flag */
  ypad = property_get_int(YPad);
  if (Apply || myApply)
    gtk_misc_set_padding(GTK_MISC(widget), xpad, ypad);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkArrow, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_arrow_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkArrow. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_arrow_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkArrow.
 * Note: as with load(), most work will already have been done.
 */
void
gb_arrow_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbDirectionValues) / sizeof(GbDirectionValues[0]);
       i++) {
    if (GbDirectionValues[i] == GTK_ARROW(widget)->arrow_type)
      save_choice(data, Direction, GbDirectionChoices[i]);
  }

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_ARROW(widget)->shadow_type)
      save_choice(data, Shadow, GbShadowChoices[i]);
  }
  save_float(data, XAlign, GTK_MISC(widget)->xalign);
  save_float(data, YAlign, GTK_MISC(widget)->yalign);
  save_int(data, XPad, GTK_MISC(widget)->xpad);
  save_int(data, YPad, GTK_MISC(widget)->ypad);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_arrow_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_arrow_init()
{
  /* Initialise the GTK type */
  gtk_arrow_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = arrow_xpm;
  gbwidget.tooltip = "Arrow";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_create_properties	= gb_arrow_create_properties;
  gbwidget.gb_widget_show_properties	= gb_arrow_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_arrow_apply_properties;
/*
  gbwidget.gb_widget_new		= gb_arrow_new;
  gbwidget.gb_widget_create_popup_menu	= gb_arrow_create_popup_menu;
  gbwidget.gb_widget_load		= gb_arrow_load;
*/
  gbwidget.gb_widget_save		= gb_arrow_save;
/*
  gbwidget.gb_widget_write_source	= gb_arrow_write_source;
*/

  return &gbwidget;
}

