/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/aspectframe.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Label	  = "GtkAspectFrame::label";
static gchar *LabelXAlign = "GtkAspectFrame::label_xalign";
/* Frame's yalign is never used */
/*static gchar *LabelYAlign = "GtkAspectFrame::label_yalign";*/
static gchar *Shadow	= "GtkAspectFrame::shadow";
static gchar *XAlign = "GtkAspectFrame::xalign";
static gchar *YAlign = "GtkAspectFrame::yalign";
static gchar *Ratio = "GtkAspectFrame::ratio";
static gchar *Obey = "GtkAspectFrame::obey_child";

static gchar* GbShadowChoices[] = { "None", "In", "Out",
				    "Etched In", "Etched Out", NULL};
static gint GbShadowValues[] = {
  GTK_SHADOW_NONE,
  GTK_SHADOW_IN,
  GTK_SHADOW_OUT,
  GTK_SHADOW_ETCHED_IN,
  GTK_SHADOW_ETCHED_OUT
};


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkAspectFrame, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_aspect_frame_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_aspect_frame_new(NULL, 0.5, 0.5, 1.0, TRUE);
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_aspect_frame_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_string(Label, "Label:", "The text to display");
  property_add_float_range(LabelXAlign, "Label X Align:",
			   "The horizontal alignment of the frame's label",
			   0, 1, 0.01, 1, 0, 2);
  property_add_choice(Shadow, "Shadow:", "The type of shadow of the frame",
		      GbShadowChoices);
  property_add_float_range(XAlign, "X Align:",
			   "The horizontal alignment of the frame's child",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(YAlign, "Y Align:",
			   "The horizontal alignment of the frame's child",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float(Ratio, "Ratio:",
		     "The aspect ratio of the frame's child");
  property_add_bool(Obey, "Obey Child:",
		    "If the aspect ratio should be determined by the child");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_aspect_frame_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  property_set_string(Label, GTK_FRAME(widget)->label);
  property_set_float(LabelXAlign, GTK_FRAME(widget)->label_xalign);

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_FRAME(widget)->shadow_type)
      property_set_choice(Shadow, i);
  }

  property_set_float(XAlign, GTK_ASPECT_FRAME(widget)->xalign);
  property_set_float(YAlign, GTK_ASPECT_FRAME(widget)->yalign);
  property_set_float(Ratio, GTK_ASPECT_FRAME(widget)->ratio);
  property_set_bool(Obey, GTK_ASPECT_FRAME(widget)->obey_child);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_aspect_frame_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *label;
  gfloat label_xalign, xalign, yalign, ratio;
  gchar *shadow;
  gboolean obey_child, myApply;
  gint i;

  label = property_get_string(Label);
  if (Apply) gtk_frame_set_label(GTK_FRAME(widget), label);

  label_xalign = property_get_float(LabelXAlign);
  if (Apply) gtk_frame_set_label_align(GTK_FRAME(widget), label_xalign, 0.5);

  shadow = property_get_choice(Shadow);
  if (Apply) {
    for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
      if (!strcmp(shadow, GbShadowChoices[i])) {
	gtk_frame_set_shadow_type(GTK_FRAME(widget), GbShadowValues[i]);
	break;
      }
    }
  }

  xalign = property_get_float(XAlign);
  myApply = Apply;
  yalign = property_get_float(YAlign);
  myApply |= Apply;
  ratio = property_get_float(Ratio);
  myApply |= Apply;
  obey_child = property_get_bool(Obey);
  myApply |= Apply;
  if (myApply) gtk_aspect_frame_set(GTK_ASPECT_FRAME(widget), xalign, yalign,
				    ratio, obey_child);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkAspectFrame, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_aspect_frame_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkAspectFrame. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_aspect_frame_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkAspectFrame.
 * Note: as with load(), most work will already have been done.
 */
void
gb_aspect_frame_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  save_string(data, Label, GTK_FRAME(widget)->label);
  save_float(data, LabelXAlign, GTK_FRAME(widget)->label_xalign);

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_FRAME(widget)->shadow_type)
      save_choice(data, Shadow, GbShadowChoices[i]);
  }

  save_float(data, XAlign, GTK_ASPECT_FRAME(widget)->xalign);
  save_float(data, YAlign, GTK_ASPECT_FRAME(widget)->yalign);
  save_float(data, Ratio, GTK_ASPECT_FRAME(widget)->ratio);
  save_bool(data, Obey, GTK_ASPECT_FRAME(widget)->obey_child);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_aspect_frame_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_aspect_frame_init()
{
  /* Initialise the GTK type */
  gtk_aspect_frame_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = aspectframe_xpm;
  gbwidget.tooltip = "Aspect Frame";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_aspect_frame_new;
  gbwidget.gb_widget_create_properties	= gb_aspect_frame_create_properties;
  gbwidget.gb_widget_show_properties	= gb_aspect_frame_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_aspect_frame_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_aspect_frame_create_popup_menu;
  gbwidget.gb_widget_load		= gb_aspect_frame_load;
*/
  gbwidget.gb_widget_save		= gb_aspect_frame_save;
/*
  gbwidget.gb_widget_write_source	= gb_aspect_frame_write_source;
*/

  return &gbwidget;
}

