/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/clist.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Mode	= "GtkCList::selection_mode";
static gchar *Titles	= "GtkCList::show_titles";
static gchar *Shadow	= "GtkCList::shadow";
static gchar *HPolicy	= "GtkCList::hpolicy";
static gchar *VPolicy	= "GtkCList::vpolicy";

static gchar* GbModeChoices[] = { "Single", "Browse", "Multiple",
				     "Extended", NULL};
static gint GbModeValues[] = {
  GTK_SELECTION_SINGLE,
  GTK_SELECTION_BROWSE,
  GTK_SELECTION_MULTIPLE,
  GTK_SELECTION_EXTENDED
};

static gchar* GbShadowChoices[] = { "None", "In", "Out",
				    "Etched In", "Etched Out", NULL};
static gint GbShadowValues[] = {
  GTK_SHADOW_NONE,
  GTK_SHADOW_IN,
  GTK_SHADOW_OUT,
  GTK_SHADOW_ETCHED_IN,
  GTK_SHADOW_ETCHED_OUT
};

static gchar* GbPolicyChoices[] = { "Always", "Automatic", NULL };
static gint GbPolicyValues[] = {
  GTK_POLICY_ALWAYS,
  GTK_POLICY_AUTOMATIC
};


static void show_clist_dialog(GbWidgetNewData *data);
static void on_clist_dialog_ok(GtkWidget *widget, GbWidgetNewData *data);
static void on_clist_dialog_destroy(GtkWidget *widget, GbWidgetNewData *data);
static GtkWidget* new_label();

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkCList, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_clist_new(GbWidgetNewData *data)
{
  show_clist_dialog(data);
  return NULL;
}


void
show_clist_dialog(GbWidgetNewData *data)
{
  GtkWidget *window, *vbox, *hbox, *separator, *hbbox, *label, *spinbutton;
  GtkWidget *button;
  GtkObject *adjustment;

  window = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
  gtk_window_set_title(GTK_WINDOW(window), "New columned list");

  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC(on_clist_dialog_destroy),
		      data);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10);*/
  gtk_widget_show(vbox);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_widget_show(hbox);

  label = gtk_label_new("Number of columns:");
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
  gtk_widget_show(label);

  adjustment = gtk_adjustment_new(3, 1, 100, 1, 10, 10);
  spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
  /* save pointer to spinbutton so we can find it easily in the OK handler */
  gtk_object_set_data(GTK_OBJECT(window), "spinbutton", spinbutton);
  gtk_box_pack_start(GTK_BOX(hbox), spinbutton, TRUE, TRUE, 5);
  gtk_widget_set_usize(spinbutton, 50, -1);
  gtk_widget_grab_focus(spinbutton);
  gtk_widget_show(spinbutton);

  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 5);
  gtk_widget_show(separator);

  hbbox = gtk_hbutton_box_new();
  gtk_box_pack_start(GTK_BOX(vbox), hbbox, TRUE, TRUE, 5);
  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show(hbbox);

  button = gtk_button_new_with_label("OK");
  gtk_container_add(GTK_CONTAINER(hbbox), button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  gtk_widget_show(button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(on_clist_dialog_ok),
		      data);

  button = gtk_button_new_with_label("Cancel");
  gtk_container_add(GTK_CONTAINER(hbbox), button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_show(button);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (window));

  gtk_widget_show(window);
  gtk_grab_add(window);
}


void
on_clist_dialog_ok(GtkWidget *widget, GbWidgetNewData *data)
{
  GtkWidget *new_widget, *spinbutton, *window;
  gint cols, i;

  window = gtk_widget_get_toplevel(widget);

  /* Only call callback if placeholder/fixed widget is still there */
  if (data->widget->parent) {
    spinbutton = gtk_object_get_data(GTK_OBJECT(window), "spinbutton");
    g_return_if_fail(spinbutton != NULL);
    cols = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));

    new_widget = gtk_clist_new(cols);
    gtk_clist_column_titles_show(GTK_CLIST(new_widget));
    for (i = 0; i < cols; i++) {
      gtk_clist_set_column_widget(GTK_CLIST(new_widget), i, new_label());
      gtk_clist_set_column_width(GTK_CLIST(new_widget), i, 80);
    }
    new_widget = gb_widget_initialize(new_widget, data);
    (*data->callback)(new_widget, data);
  }
  gtk_widget_destroy(window);
}


void
on_clist_dialog_destroy(GtkWidget *widget, GbWidgetNewData *data)
{
  gb_widget_free_new_data(data);
  gtk_grab_remove(widget);
}


GtkWidget*
new_label()
{
  GbWidgetNewData *data;
  GtkWidget *label;

  data = g_new(GbWidgetNewData, 1);
  data->widget = NULL;
  label = gb_widget_new("GtkLabel", data);
  gb_widget_free_new_data(data);
  g_return_val_if_fail(label != NULL, NULL);
  return label;
}


/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_clist_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Mode, "Select Mode:",
		      "The selection mode of the columned list",
		      GbModeChoices);
  property_add_bool(Titles, "Show Titles:",
		    "If the column titles are shown");
  property_add_choice(Shadow, "Shadow:",
		      "The type of shadow of the columned list's border",
		      GbShadowChoices);
  property_add_choice(HPolicy, "H Policy:",
		      "When the horizontal scrollbar will be shown",
		      GbPolicyChoices);
  property_add_choice(VPolicy, "V Policy:",
		      "When the vertical scrollbar will be shown",
		      GbPolicyChoices);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_clist_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]); i++) {
    if (GbModeValues[i] == GTK_CLIST(widget)->selection_mode)
      property_set_choice(Mode, i);
  }

  property_set_bool(Titles, GTK_CLIST_SHOW_TITLES(widget));

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_CLIST(widget)->shadow_type)
      property_set_choice(Shadow, i);
  }

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_CLIST(widget)->hscrollbar_policy)
      property_set_choice(HPolicy, i);
  }
  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_CLIST(widget)->vscrollbar_policy)
      property_set_choice(VPolicy, i);
  }
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_clist_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *mode;
  gchar *shadow, *hpolicy, *vpolicy;
  gboolean myApply, titles;
  gint i;
  GtkPolicyType hpolicy_value = GTK_POLICY_AUTOMATIC;
  GtkPolicyType vpolicy_value = GTK_POLICY_AUTOMATIC;

  mode = property_get_choice(Mode);
  if (Apply) {
    for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]);i++) {
      if (!strcmp(mode, GbModeChoices[i])) {
	gtk_clist_set_selection_mode(GTK_CLIST(widget), GbModeValues[i]);
	break;
      }
    }
  }

  titles = property_get_bool(Titles);
  if (Apply) {
    if (titles) gtk_clist_column_titles_show(GTK_CLIST(widget));
    else gtk_clist_column_titles_hide(GTK_CLIST(widget));
  }

  shadow = property_get_choice(Shadow);
  if (Apply) {
    for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]);i++) {
      if (!strcmp(shadow, GbShadowChoices[i])) {
	gtk_clist_set_border(GTK_CLIST(widget), GbShadowValues[i]);
	break;
      }
    }
  }

  hpolicy = property_get_choice(HPolicy);
  myApply = Apply;
  vpolicy = property_get_choice(VPolicy);
  if (Apply || myApply) {
    for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]);i++) {
      if (!strcmp(hpolicy, GbPolicyChoices[i]))
	hpolicy_value = GbPolicyValues[i];
      if (!strcmp(vpolicy, GbPolicyChoices[i]))
	vpolicy_value = GbPolicyValues[i];
    }
    gtk_clist_set_policy(GTK_CLIST(widget), hpolicy_value, vpolicy_value);
  }
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkCList, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_clist_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkCList. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_clist_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkCList.
 * Note: as with load(), most work will already have been done.
 */
void
gb_clist_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]); i++) {
    if (GbModeValues[i] == GTK_CLIST(widget)->selection_mode)
      save_choice(data, Mode, GbModeChoices[i]);
  }

  save_bool(data, Titles, GTK_CLIST_SHOW_TITLES(widget));

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_CLIST(widget)->shadow_type)
      save_choice(data, Shadow, GbShadowChoices[i]);
  }

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_CLIST(widget)->hscrollbar_policy)
      save_choice(data, HPolicy, GbPolicyChoices[i]);
  }
  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_CLIST(widget)->vscrollbar_policy)
      save_choice(data, VPolicy, GbPolicyChoices[i]);
  }
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_clist_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_clist_init()
{
  /* Initialise the GTK type */
  gtk_clist_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = clist_xpm;
  gbwidget.tooltip = "Columned List";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_clist_new;
  gbwidget.gb_widget_create_properties	= gb_clist_create_properties;
  gbwidget.gb_widget_show_properties	= gb_clist_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_clist_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_clist_create_popup_menu;
  gbwidget.gb_widget_load		= gb_clist_load;
*/
  gbwidget.gb_widget_save		= gb_clist_save;
/*
  gbwidget.gb_widget_write_source	= gb_clist_write_source;
*/

  return &gbwidget;
}

