/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/colorselection.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Policy	= "GtkColorSelection::policy";

static gchar* GbPolicyChoices[] = { "Continuous", "Discontinuous", "Delayed",
				     NULL };
static gint GbPolicyValues[] = {
  GTK_UPDATE_CONTINUOUS,
  GTK_UPDATE_DISCONTINUOUS,
  GTK_UPDATE_DELAYED
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkColorSelection, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
/*
GtkWidget*
gb_color_selection_new(GbWidgetNewData *data)
{

}
*/



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_color_selection_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Policy, "Update Policy:",
		      "The update policy of the color selection scales",
		      GbPolicyChoices);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_color_selection_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_COLOR_SELECTION(widget)->policy)
      property_set_choice(Policy, i);
  }
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_color_selection_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gint i;
  gchar *policy;

  policy = property_get_choice(Policy);
  if (Apply) {
    for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]);i++) {
      if (!strcmp(policy, GbPolicyChoices[i])) {
	gtk_color_selection_set_update_policy(GTK_COLOR_SELECTION(widget),
					      GbPolicyValues[i]);
	break;
      }
    }
  }
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkColorSelection, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_color_selection_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkColorSelection. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_color_selection_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkColorSelection.
 * Note: as with load(), most work will already have been done.
 */
void
gb_color_selection_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_COLOR_SELECTION(widget)->policy)
      save_choice(data, Policy, GbPolicyChoices[i]);
  }
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_color_selection_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_color_selection_init()
{
  /* Initialise the GTK type */
  gtk_color_selection_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = colorselection_xpm;
  gbwidget.tooltip = "Color Selection";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_create_properties	= gb_color_selection_create_properties;
  gbwidget.gb_widget_show_properties	= gb_color_selection_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_color_selection_apply_properties;
/*
  gbwidget.gb_widget_new		= gb_color_selection_new;
  gbwidget.gb_widget_create_popup_menu	= gb_color_selection_create_popup_menu;
  gbwidget.gb_widget_load		= gb_color_selection_load;
*/
  gbwidget.gb_widget_save		= gb_color_selection_save;
/*
  gbwidget.gb_widget_write_source	= gb_color_selection_write_source;
*/

  return &gbwidget;
}

