/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/curve.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Type	= "GtkCurve::type";
static gchar *XMin	= "GtkCurve::xmin";
static gchar *XMax	= "GtkCurve::xmax";
static gchar *YMin	= "GtkCurve::ymin";
static gchar *YMax	= "GtkCurve::ymax";

static gchar* GbTypeChoices[] = { "Linear", "Spline", "Free", NULL };
static gint GbTypeValues[] = {
  GTK_CURVE_TYPE_LINEAR,	/* linear interpolation */
  GTK_CURVE_TYPE_SPLINE,	/* spline interpolation */
  GTK_CURVE_TYPE_FREE		/* free form curve */
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkCurve, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_curve_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_curve_new();
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_curve_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Type, "Type:", "The type of the curve", GbTypeChoices);
  property_add_float(XMin, "X Min:", "The minimum horizontal value");
  property_add_float(XMax, "X Max:", "The maximum horizontal value");
  property_add_float(YMin, "Y Min:", "The minimum vertical value");
  property_add_float(YMax, "Y Max:", "The maximum vertical value");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_curve_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbTypeValues) / sizeof(GbTypeValues[0]); i++) {
    if (GbTypeValues[i] == GTK_CURVE(widget)->curve_type)
      property_set_choice(Type, i);
  }
  property_set_float(XMin, GTK_CURVE(widget)->min_x);
  property_set_float(XMax, GTK_CURVE(widget)->max_x);
  property_set_float(YMin, GTK_CURVE(widget)->min_y);
  property_set_float(YMax, GTK_CURVE(widget)->max_y);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_curve_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gint i;
  gchar *type;
  gfloat min_x, max_x, min_y, max_y;
  gboolean myApply;

  type = property_get_choice(Type);
  if (Apply) {
    for (i = 0; i < sizeof(GbTypeValues) / sizeof(GbTypeValues[0]);i++) {
      if (!strcmp(type, GbTypeChoices[i])) {
	gtk_curve_set_curve_type(GTK_CURVE(widget), GbTypeValues[i]);
	break;
      }
    }
  }

  min_x = property_get_float(XMin);
  myApply = Apply;
  max_x = property_get_float(XMax);
  myApply |= Apply;
  min_y = property_get_float(YMin);
  myApply |= Apply;
  max_y = property_get_float(YMax);
  myApply |= Apply;
  if (myApply) gtk_curve_set_range(GTK_CURVE(widget), min_x, max_x,
				   min_y, max_y);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkCurve, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_curve_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkCurve. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_curve_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkCurve.
 * Note: as with load(), most work will already have been done.
 */
void
gb_curve_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbTypeValues) / sizeof(GbTypeValues[0]); i++) {
    if (GbTypeValues[i] == GTK_CURVE(widget)->curve_type)
      save_choice(data, Type, GbTypeChoices[i]);
  }
  save_float(data, XMin, GTK_CURVE(widget)->min_x);
  save_float(data, XMax, GTK_CURVE(widget)->max_x);
  save_float(data, YMin, GTK_CURVE(widget)->min_y);
  save_float(data, YMax, GTK_CURVE(widget)->max_y);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_curve_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_curve_init()
{
  /* Initialise the GTK type */
  gtk_curve_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = curve_xpm;
  gbwidget.tooltip = "Curve";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_curve_new;
  gbwidget.gb_widget_create_properties	= gb_curve_create_properties;
  gbwidget.gb_widget_show_properties	= gb_curve_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_curve_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_curve_create_popup_menu;
  gbwidget.gb_widget_load		= gb_curve_load;
*/
  gbwidget.gb_widget_save		= gb_curve_save;
/*
  gbwidget.gb_widget_write_source	= gb_curve_write_source;
*/

  return &gbwidget;
}

