/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/entry.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Editable	= "GtkEntry::editable";
static gchar *Visible	= "GtkEntry::visible";
static gchar *MaxLength	= "GtkEntry::max_length";


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkEntry, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
/*
GtkWidget*
gb_entry_new(GbWidgetNewData *data)
{

}
*/



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_entry_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_bool(Editable, "Editable:", "If the text can be edited");
  property_add_bool(Visible, "Visible:", "If the text entered will be shown");
  property_add_int_range(MaxLength, "Max Length:",
			 "The maximum length of the text",
			 0, 10000, 1, 10, 1);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_entry_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  property_set_bool(Editable, GTK_EDITABLE(widget)->editable);
  property_set_bool(Visible, GTK_ENTRY(widget)->visible);
  property_set_int(MaxLength, GTK_ENTRY(widget)->text_max_length);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_entry_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gboolean editable, visible;
  gint max_length;

  editable = property_get_bool(Editable);
  if (Apply) gtk_entry_set_editable(GTK_ENTRY(widget), editable);

  visible = property_get_bool(Visible);
  if (Apply) gtk_entry_set_visibility(GTK_ENTRY(widget), visible);

  max_length = property_get_int(MaxLength);
  if (Apply) gtk_entry_set_max_length(GTK_ENTRY(widget), max_length);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkEntry, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_entry_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkEntry. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_entry_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkEntry.
 * Note: as with load(), most work will already have been done.
 */
void
gb_entry_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  save_bool(data, Editable, GTK_EDITABLE(widget)->editable);
  save_bool(data, Visible, GTK_ENTRY(widget)->visible);
  save_int(data, MaxLength, GTK_ENTRY(widget)->text_max_length);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_entry_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_entry_init()
{
  /* Initialise the GTK type */
  gtk_entry_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = entry_xpm;
  gbwidget.tooltip = "Text Entry";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_create_properties	= gb_entry_create_properties;
  gbwidget.gb_widget_show_properties	= gb_entry_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_entry_apply_properties;
/*
  gbwidget.gb_widget_new		= gb_entry_new;
  gbwidget.gb_widget_create_popup_menu	= gb_entry_create_popup_menu;
  gbwidget.gb_widget_load		= gb_entry_load;
*/
  gbwidget.gb_widget_save		= gb_entry_save;
/*
  gbwidget.gb_widget_write_source	= gb_entry_write_source;
*/

  return &gbwidget;
}

