/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
/* This widget has no icon as it doesn't appear in the palette */
/*#include "../graphics/"*/

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Title	= "GtkFileSel::title";
static gchar *Position	= "GtkFileSel::position";
static gchar *Shrink	= "GtkFileSel::shrink";
static gchar *Grow	= "GtkFileSel::grow";
static gchar *AutoShrink= "GtkFileSel::auto_shrink";

static gchar* GbPositionChoices[] = { "None", "Center", "Mouse", NULL };
static gint GbPositionValues[] = {
  GTK_WIN_POS_NONE,
  GTK_WIN_POS_CENTER,
  GTK_WIN_POS_MOUSE
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkFileSelection, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_file_selection_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_file_selection_new("Select File");
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_file_selection_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_string(Title, "Title:", "The title of the window");
  property_add_choice(Position, "Position:",
		      "The initial position of the dialog",
		      GbPositionChoices);
  property_add_bool(Shrink, "Shrink:", "If the window can be shrunk");
  property_add_bool(Grow, "Grow:", "If the window can be enlarged");
  property_add_bool(AutoShrink, "Auto Shrink:",
		    "If the window shrinks automatically");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_file_selection_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  property_set_string(Title, GTK_WINDOW(widget)->title);
  for (i = 0; i < sizeof(GbPositionValues) / sizeof(GbPositionValues[0]); i++){
    if (GbPositionValues[i] == GTK_WINDOW(widget)->position)
      property_set_choice(Position, i);
  }
  property_set_bool(Shrink, GTK_WINDOW(widget)->allow_shrink);
  property_set_bool(Grow, GTK_WINDOW(widget)->allow_grow);
  property_set_bool(AutoShrink, GTK_WINDOW(widget)->auto_shrink);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_file_selection_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *title, *position;
  gboolean shrink, grow, auto_shrink, myApply;
  gint i;

  title = property_get_string(Title);
  if (Apply) gtk_window_set_title(GTK_WINDOW(widget), title);

  position = property_get_choice(Position);
  if (Apply) {
    for (i = 0; i < sizeof(GbPositionValues)/ sizeof(GbPositionValues[0]);i++){
      if (!strcmp(position, GbPositionChoices[i])) {
	gtk_window_position(GTK_WINDOW(widget), GbPositionValues[i]);
	break;
      }
    }
  }

  shrink = property_get_bool(Shrink);
  myApply = Apply;
  grow = property_get_bool(Grow);
  myApply |= Apply;
  auto_shrink = property_get_bool(AutoShrink);
  if (Apply || myApply) gtk_window_set_policy(GTK_WINDOW(widget), shrink, grow,
					      auto_shrink);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkFileSelection, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_file_selection_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkFileSelection. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_file_selection_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkFileSelection.
 * Note: as with load(), most work will already have been done.
 */
void
gb_file_selection_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  save_string(data, Title, GTK_WINDOW(widget)->title);
  for (i = 0; i < sizeof(GbPositionValues) / sizeof(GbPositionValues[0]); i++){
    if (GbPositionValues[i] == GTK_WINDOW(widget)->position)
      save_choice(data, Position, GbPositionChoices[i]);
  }
  save_bool(data, Shrink, GTK_WINDOW(widget)->allow_shrink);
  save_bool(data, Grow, GTK_WINDOW(widget)->allow_grow);
  save_bool(data, AutoShrink, GTK_WINDOW(widget)->auto_shrink);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_file_selection_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_file_selection_init()
{
  /* Initialise the GTK type */
  gtk_file_selection_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = NULL;
  gbwidget.tooltip = NULL;

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_file_selection_new;
  gbwidget.gb_widget_create_properties	= gb_file_selection_create_properties;
  gbwidget.gb_widget_show_properties	= gb_file_selection_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_file_selection_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_file_selection_create_popup_menu;
  gbwidget.gb_widget_load		= gb_file_selection_load;
*/
  gbwidget.gb_widget_save		= gb_file_selection_save;
/*
  gbwidget.gb_widget_write_source	= gb_file_selection_write_source;
*/

  return &gbwidget;
}

