/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
/* This widget has no icon as it doesn't appear in the palette */
/*#include "../graphics/"*/

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Label	= "GtkMenuItem::label";
static gchar *Justify	= "GtkMenuItem::justify";


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkMenuItem, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_menu_item_new(GbWidgetNewData *data)
{
  return gtk_menu_item_new_with_label(data->name);
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_menu_item_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_text(Label, "Label:", "The text to display", 2);
  property_add_bool(Justify, "Right Justify:",
		    "If the menu item is right-justified");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_menu_item_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  if (GTK_IS_LABEL(GTK_BUTTON(widget)->child)) {
    gchar *label_text;
    gtk_label_get(GTK_LABEL(GTK_BUTTON(widget)->child), &label_text);
    property_set_text(Label, label_text);
    property_set_sensitive(Label, TRUE);
  } else {
    property_set_text(Label, "");
    property_set_sensitive(Label, FALSE);
  }
  property_set_bool(Justify, GTK_MENU_ITEM(widget)->right_justify);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_menu_item_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gboolean justify;
  
  if (GTK_IS_LABEL(GTK_BUTTON(widget)->child)) {
    gchar *label_text = property_get_text(Label);
    if (Apply) gtk_label_set(GTK_LABEL(GTK_BUTTON(widget)->child), label_text);
    g_free(label_text);
  }

  justify = property_get_bool(Justify);
  /* GtkMenuItem doesn't have a function for setting/clearing right-justify */
  if (Apply) GTK_MENU_ITEM(widget)->right_justify = justify;
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkMenuItem, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_menu_item_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkMenuItem. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_menu_item_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkMenuItem.
 * Note: as with load(), most work will already have been done.
 */
void
gb_menu_item_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  if (GTK_IS_LABEL(GTK_BUTTON(widget)->child)) {
    gchar *label_text;
    gtk_label_get(GTK_LABEL(GTK_BUTTON(widget)->child), &label_text);
    save_text(data, Label, label_text);
  } else {
    save_text(data, Label, "");
  }
  save_bool(data, Justify, GTK_MENU_ITEM(widget)->right_justify);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_menu_item_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_menu_item_init()
{
  /* Initialise the GTK type */
  gtk_menu_item_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = NULL;
  gbwidget.tooltip = NULL;

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_menu_item_new;
  gbwidget.gb_widget_create_properties	= gb_menu_item_create_properties;
  gbwidget.gb_widget_show_properties	= gb_menu_item_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_menu_item_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_menu_item_create_popup_menu;
  gbwidget.gb_widget_load		= gb_menu_item_load;
*/
  gbwidget.gb_widget_save		= gb_menu_item_save;
/*
  gbwidget.gb_widget_write_source	= gb_menu_item_write_source;
*/

  return &gbwidget;
}

