/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/pixmap.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Filename	= "GtkPixmap::filename";
static gchar *XAlign	= "GtkPixmap::xalign";
static gchar *YAlign	= "GtkPixmap::yalign";
static gchar *XPad	= "GtkPixmap::xpad";
static gchar *YPad	= "GtkPixmap::ypad";

static gchar *GbFilenameKey = "GbFilename";

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkPixmap, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_pixmap_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget;
  new_widget = gtk_pixmap_new(gbwidget.gdkpixmap, gbwidget.mask);
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_pixmap_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_filename(Filename, "File:", "The pixmap filename");
  property_add_float_range(XAlign, "X Align:",
			   "The horizontal alignment",
			   0, 1, 0.01, 1, 0, 2);
  property_add_float_range(YAlign, "Y Align:",
			   "The vertical alignment",
			   0, 1, 0.01, 1, 0, 2);
  property_add_int_range(XPad, "X Pad:", "The horizontal padding",
			 0, 1000, 1, 10, 1);
  property_add_int_range(YPad, "Y Pad:", "The vertical padding",
			 0, 1000, 1, 10, 1);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_pixmap_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gchar *filename = gtk_object_get_data(GTK_OBJECT(widget), GbFilenameKey);
  property_set_filename(Filename, filename);
  property_set_float(XAlign, GTK_MISC(widget)->xalign);
  property_set_float(YAlign, GTK_MISC(widget)->yalign);
  property_set_int(XPad, GTK_MISC(widget)->xpad);
  property_set_int(YPad, GTK_MISC(widget)->ypad);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_pixmap_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *filename;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  gfloat xalign, yalign;
  gint xpad, ypad;
  gboolean myApply;

  filename = property_get_filename(Filename);
  if (Apply) {
    gtk_object_set_data(GTK_OBJECT(widget), GbFilenameKey, g_strdup(filename));

    gdkpixmap = gdk_pixmap_create_from_xpm (widget->window, &mask,
					  &widget->style->bg[GTK_STATE_NORMAL],
					    filename);
    if (!gdkpixmap) {
      show_message_box("Couldn't create pixmap from file\n");
    } else {
      gtk_pixmap_set(GTK_PIXMAP(widget), gdkpixmap, mask);
      editor_refresh_widget(widget);
    }
  }

  xalign = property_get_float(XAlign);
  myApply = Apply; /* save Apply flag */
  yalign = property_get_float(YAlign);
  if (Apply || myApply)
    gtk_misc_set_alignment(GTK_MISC(widget), xalign, yalign);

  xpad = property_get_int(XPad);
  myApply = Apply; /* save Apply flag */
  ypad = property_get_int(YPad);
  if (Apply || myApply)
    gtk_misc_set_padding(GTK_MISC(widget), xpad, ypad);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkPixmap, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_pixmap_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkPixmap. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_pixmap_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkPixmap.
 * Note: as with load(), most work will already have been done.
 */
void
gb_pixmap_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gchar *filename = gtk_object_get_data(GTK_OBJECT(widget), GbFilenameKey);
  save_filename(data, Filename, filename);
  save_float(data, XAlign, GTK_MISC(widget)->xalign);
  save_float(data, YAlign, GTK_MISC(widget)->yalign);
  save_int(data, XPad, GTK_MISC(widget)->xpad);
  save_int(data, YPad, GTK_MISC(widget)->ypad);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_pixmap_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_pixmap_init()
{
  /* Initialise the GTK type */
  gtk_pixmap_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = pixmap_xpm;
  gbwidget.tooltip = "Pixmap";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_pixmap_new;
  gbwidget.gb_widget_create_properties	= gb_pixmap_create_properties;
  gbwidget.gb_widget_show_properties	= gb_pixmap_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_pixmap_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_pixmap_create_popup_menu;
  gbwidget.gb_widget_load		= gb_pixmap_load;
*/
  gbwidget.gb_widget_save		= gb_pixmap_save;
/*
  gbwidget.gb_widget_write_source	= gb_pixmap_write_source;
*/

  return &gbwidget;
}

