/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/scrolledwindow.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Shadow		= "GtkScrolledWindow::shadow";
static gchar *HPolicy		= "GtkScrolledWindow::hpolicy";
static gchar *VPolicy		= "GtkScrolledWindow::vpolicy";
static gchar *HUpdatePolicy	= "GtkScrolledWindow::h_update_policy";
static gchar *VUpdatePolicy	= "GtkScrolledWindow::v_update_policy";

static gchar *HValues[]	= {
  "GtkScrolledWindow::hvalue",
  "GtkScrolledWindow::hlower",
  "GtkScrolledWindow::hupper",
  "GtkScrolledWindow::hstep",
  "GtkScrolledWindow::hpage",
  "GtkScrolledWindow::hpage_size",
};

static gchar *VValues[]	= {
  "GtkScrolledWindow::vvalue",
  "GtkScrolledWindow::vlower",
  "GtkScrolledWindow::vupper",
  "GtkScrolledWindow::vstep",
  "GtkScrolledWindow::vpage",
  "GtkScrolledWindow::vpage_size",
};

static gchar* GbShadowChoices[] = { "None", "In", "Out",
				    "Etched In", "Etched Out", NULL};
static gint GbShadowValues[] = {
  GTK_SHADOW_NONE,
  GTK_SHADOW_IN,
  GTK_SHADOW_OUT,
  GTK_SHADOW_ETCHED_IN,
  GTK_SHADOW_ETCHED_OUT
};

static gchar* GbPolicyChoices[] = { "Always", "Automatic", NULL };
static gint GbPolicyValues[] = {
  GTK_POLICY_ALWAYS,
  GTK_POLICY_AUTOMATIC
};

static gchar* GbUpdatePolicyChoices[] = { "Continuous", "Discontinuous",
					  "Delayed", NULL };
static gint GbUpdatePolicyValues[] = {
  GTK_UPDATE_CONTINUOUS,
  GTK_UPDATE_DISCONTINUOUS,
  GTK_UPDATE_DELAYED
};


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkScrolledWindow, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_scrolled_window_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_scrolled_window_new(NULL, NULL);
  /*add_draw_signals(GTK_SCROLLED_WINDOW(new_widget)->hscrollbar, NULL);*/
  /*add_draw_signals(GTK_SCROLLED_WINDOW(new_widget)->vscrollbar, NULL);*/
  /*add_draw_signals(GTK_SCROLLED_WINDOW(new_widget)->viewport, NULL);*/
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_scrolled_window_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Shadow, "Shadow:", "The type of shadow of the viewport",
		      GbShadowChoices);
  property_add_choice(HPolicy, "H Policy:",
		      "When the horizontal scrollbar will be shown",
		      GbPolicyChoices);
  property_add_choice(VPolicy, "V Policy:",
		      "When the vertical scrollbar will be shown",
		      GbPolicyChoices);
  property_add_choice(HUpdatePolicy, "H Updates:",
		      "The update policy of the horizontal scrollbar",
		      GbUpdatePolicyChoices);
  property_add_choice(VUpdatePolicy, "V Updates:",
		      "The update policy of the vertical scrollbar",
		      GbUpdatePolicyChoices);
  property_add_adjustment(HValues, GB_ADJUST_H_LABELS);
  property_add_adjustment(VValues, GB_ADJUST_V_LABELS);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_scrolled_window_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;
  GtkViewport *viewport = GTK_VIEWPORT(GTK_SCROLLED_WINDOW(widget)->viewport);

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == viewport->shadow_type)
      property_set_choice(Shadow, i);
  }

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_SCROLLED_WINDOW(widget)->hscrollbar_policy)
      property_set_choice(HPolicy, i);
  }
  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_SCROLLED_WINDOW(widget)->vscrollbar_policy)
      property_set_choice(VPolicy, i);
  }

  for (i = 0; i < sizeof(GbUpdatePolicyValues)
	 / sizeof(GbUpdatePolicyValues[0]); i++) {
    if (GbUpdatePolicyValues[i] == GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
	->hscrollbar)->policy)
      property_set_choice(HUpdatePolicy, i);
  }
  for (i = 0; i < sizeof(GbUpdatePolicyValues)
	 / sizeof(GbUpdatePolicyValues[0]); i++) {
    if (GbUpdatePolicyValues[i] == GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
	->vscrollbar)->policy)
      property_set_choice(VUpdatePolicy, i);
  }

  property_show_adjustment(HValues, viewport->hadjustment);
  property_show_adjustment(VValues, viewport->vadjustment);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_scrolled_window_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gint i;
  gchar *shadow, *hpolicy, *vpolicy, *hupdate_policy, *vupdate_policy;
  gboolean myApply;
  GtkPolicyType hpolicy_value = GTK_POLICY_AUTOMATIC;
  GtkPolicyType vpolicy_value = GTK_POLICY_AUTOMATIC;
  GtkUpdateType hupdate_policy_value = GTK_UPDATE_CONTINUOUS;
  GtkUpdateType vupdate_policy_value = GTK_UPDATE_CONTINUOUS;
  GtkViewport *viewport = GTK_VIEWPORT(GTK_SCROLLED_WINDOW(widget)->viewport);

  shadow = property_get_choice(Shadow);
  if (Apply) {
    for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]);i++) {
      if (!strcmp(shadow, GbShadowChoices[i])) {
	gtk_viewport_set_shadow_type(viewport, GbShadowValues[i]);
	break;
      }
    }
  }

  hpolicy = property_get_choice(HPolicy);
  myApply = Apply;
  vpolicy = property_get_choice(VPolicy);
  if (Apply || myApply) {
    for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]);i++) {
      if (!strcmp(hpolicy, GbPolicyChoices[i]))
	hpolicy_value = GbPolicyValues[i];
      if (!strcmp(vpolicy, GbPolicyChoices[i]))
	vpolicy_value = GbPolicyValues[i];
    }
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget), hpolicy_value,
				   vpolicy_value);
  }

  hupdate_policy = property_get_choice(HUpdatePolicy);
  if (Apply) {
    for (i = 0; i < sizeof(GbUpdatePolicyValues)
	   / sizeof(GbUpdatePolicyValues[0]); i++) {
      if (!strcmp(hupdate_policy, GbUpdatePolicyChoices[i])) {
	hupdate_policy_value = GbUpdatePolicyValues[i];
	break;
      }
    }
    gtk_range_set_update_policy(GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
					  ->hscrollbar), hupdate_policy_value);
  }

  vupdate_policy = property_get_choice(VUpdatePolicy);
  if (Apply) {
    for (i = 0; i < sizeof(GbUpdatePolicyValues)
	   / sizeof(GbUpdatePolicyValues[0]); i++) {
      if (!strcmp(vupdate_policy, GbUpdatePolicyChoices[i])) {
	vupdate_policy_value = GbUpdatePolicyValues[i];
	break;
      }
    }
    gtk_range_set_update_policy(GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
					  ->vscrollbar), vupdate_policy_value);
  }

  if (property_apply_adjustment(HValues, viewport->hadjustment))
    gtk_signal_emit_by_name (GTK_OBJECT (viewport->hadjustment),
			     "value_changed");
  if (property_apply_adjustment(VValues, viewport->vadjustment))
    gtk_signal_emit_by_name (GTK_OBJECT (viewport->vadjustment),
			     "value_changed");
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkScrolledWindow, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_scrolled_window_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkScrolledWindow. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_scrolled_window_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkScrolledWindow.
 * Note: as with load(), most work will already have been done.
 */
void
gb_scrolled_window_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;
  GtkViewport *viewport = GTK_VIEWPORT(GTK_SCROLLED_WINDOW(widget)->viewport);

  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == viewport->shadow_type)
      save_choice(data, Shadow, GbShadowChoices[i]);
  }

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_SCROLLED_WINDOW(widget)->hscrollbar_policy)
      save_choice(data, HPolicy, GbPolicyChoices[i]);
  }
  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_SCROLLED_WINDOW(widget)->vscrollbar_policy)
      save_choice(data, VPolicy, GbPolicyChoices[i]);
  }

  for (i = 0; i < sizeof(GbUpdatePolicyValues)
	 / sizeof(GbUpdatePolicyValues[0]); i++) {
    if (GbUpdatePolicyValues[i] == GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
	->hscrollbar)->policy)
      save_choice(data, HUpdatePolicy, GbUpdatePolicyChoices[i]);
  }
  for (i = 0; i < sizeof(GbUpdatePolicyValues)
	 / sizeof(GbUpdatePolicyValues[0]); i++) {
    if (GbUpdatePolicyValues[i] == GTK_RANGE(GTK_SCROLLED_WINDOW(widget)
	->vscrollbar)->policy)
      save_choice(data, VUpdatePolicy, GbUpdatePolicyChoices[i]);
  }

  save_adjustment(data, HValues, viewport->hadjustment);
  save_adjustment(data, VValues, viewport->vadjustment);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_scrolled_window_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_scrolled_window_init()
{
  /* Initialise the GTK type */
  gtk_scrolled_window_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = scrolledwindow_xpm;
  gbwidget.tooltip = "Scrolled Window";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_scrolled_window_new;
  gbwidget.gb_widget_create_properties	= gb_scrolled_window_create_properties;
  gbwidget.gb_widget_show_properties	= gb_scrolled_window_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_scrolled_window_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_scrolled_window_create_popup_menu;
  gbwidget.gb_widget_load		= gb_scrolled_window_load;
*/
  gbwidget.gb_widget_save		= gb_scrolled_window_save;
/*
  gbwidget.gb_widget_write_source	= gb_scrolled_window_write_source;
*/

  return &gbwidget;
}

