/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/table.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Homogeneous = "GtkTable::homogeneous";
static gchar *RowSpacing  = "GtkTable::row_spacing";
static gchar *ColSpacing  = "GtkTable::col_spacing";


static void show_table_dialog(GbWidgetNewData *data);
static void on_table_dialog_ok(GtkWidget *widget, GbWidgetNewData *data);
static void on_table_dialog_destroy(GtkWidget *widget, GbWidgetNewData *data);

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkTable, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_table_new(GbWidgetNewData *data)
{
  show_table_dialog(data);
  return NULL;
}


void
show_table_dialog(GbWidgetNewData *data)
{
  GtkWidget *window, *vbox, *separator, *hbbox, *label, *spinbutton;
  GtkWidget *button, *table;
  GtkObject *adjustment;

  window = gtk_window_new(GTK_WINDOW_DIALOG);
  gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
  gtk_window_set_title(GTK_WINDOW(window), "New table");

  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (window));
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC(on_table_dialog_destroy),
		      data);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10);*/
  gtk_widget_show(vbox);

  /* Rows label & entry */
  table = gtk_table_new(2, 2, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 5);
  gtk_container_border_width (GTK_CONTAINER (table), 4);
  gtk_widget_show(table);

  label = gtk_label_new("Number of rows:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
  gtk_widget_show(label);

  adjustment = gtk_adjustment_new(3, 1, 100, 1, 10, 10);
  spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
  /* save pointer to entry so we can find it easily in the OK handler */
  gtk_object_set_data(GTK_OBJECT(window), "spinbutton", spinbutton);
  gtk_table_attach_defaults(GTK_TABLE(table), spinbutton, 1, 2, 0, 1);
  gtk_widget_set_usize(spinbutton, 50, -1);
  gtk_widget_grab_focus(spinbutton);
  gtk_widget_show(spinbutton);

  /* Columns label & entry */
  label = gtk_label_new("Number of columns:");
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
  gtk_widget_show(label);

  adjustment = gtk_adjustment_new(3, 1, 100, 1, 10, 10);
  spinbutton = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
  /* save pointer to entry so we can find it easily in the OK handler */
  gtk_object_set_data(GTK_OBJECT(window), "spinbutton2", spinbutton);
  gtk_table_attach_defaults(GTK_TABLE(table), spinbutton, 1, 2, 1, 2);
  gtk_widget_set_usize(spinbutton, 50, -1);
  gtk_widget_show(spinbutton);

  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 5);
  gtk_widget_show(separator);

  hbbox = gtk_hbutton_box_new();
  gtk_box_pack_start(GTK_BOX(vbox), hbbox, TRUE, TRUE, 5);
  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show(hbbox);

  button = gtk_button_new_with_label("OK");
  gtk_container_add(GTK_CONTAINER(hbbox), button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  gtk_widget_show(button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC(on_table_dialog_ok),
		      data);

  button = gtk_button_new_with_label("Cancel");
  gtk_container_add(GTK_CONTAINER(hbbox), button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_show(button);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (window));

  gtk_widget_show(window);
  gtk_grab_add(window);
}


void
on_table_dialog_ok(GtkWidget *widget, GbWidgetNewData *data)
{
  GtkWidget *new_widget, *spinbutton, *window;
  gint rows, cols, row, col;

  window = gtk_widget_get_toplevel(widget);

  /* Only call callback if placeholder/fixed widget is still there */
  if (data->widget->parent) {
    spinbutton = gtk_object_get_data(GTK_OBJECT(window), "spinbutton");
    g_return_if_fail(spinbutton != NULL);
    rows = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));

    spinbutton = gtk_object_get_data(GTK_OBJECT(window), "spinbutton2");
    g_return_if_fail(spinbutton != NULL);
    cols = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton));

    new_widget = gtk_table_new(rows, cols, FALSE);
    for (row = 0; row < rows; row++) {
      for (col = 0; col < cols; col++) {
	gtk_table_attach_defaults(GTK_TABLE (new_widget),
				  editor_new_placeholder(),
				  col, col + 1, row, row + 1);
      }
    }
    new_widget = gb_widget_initialize(new_widget, data);
    (*data->callback)(new_widget, data);
  }
  gtk_widget_destroy(window);
}


void
on_table_dialog_destroy(GtkWidget *widget, GbWidgetNewData *data)
{
  gb_widget_free_new_data(data);
  gtk_grab_remove(widget);
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_table_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_bool(Homogeneous, "Homogeneous",
		    "If the children should all be the same size");
  property_add_int_range(RowSpacing, "Row Spacing:",
			 "The space between each row",
			 0, 1000, 1, 10, 1);
  property_add_int_range(ColSpacing, "Col Spacing:",
			 "The space between each column",
			 0, 1000, 1, 10, 1);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_table_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  property_set_bool(Homogeneous, GTK_TABLE(widget)->homogeneous);
  property_set_int(RowSpacing, GTK_TABLE(widget)->row_spacing);
  property_set_int(ColSpacing, GTK_TABLE(widget)->column_spacing);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_table_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gboolean homogeneous;
  gint row_spacing, column_spacing;

  homogeneous = property_get_bool(Homogeneous);
  if (Apply) {
    GTK_TABLE(widget)->homogeneous = homogeneous;
    gtk_widget_queue_resize (widget);
  }

  row_spacing = property_get_int(RowSpacing);
  if (Apply) gtk_table_set_row_spacings(GTK_TABLE(widget), row_spacing);

  column_spacing = property_get_int(ColSpacing);
  if (Apply) gtk_table_set_col_spacings(GTK_TABLE(widget), column_spacing);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkTable, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_table_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkTable. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_table_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkTable.
 * Note: as with load(), most work will already have been done.
 */
void
gb_table_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  save_bool(data, Homogeneous, GTK_TABLE(widget)->homogeneous);
  save_int(data, RowSpacing, GTK_TABLE(widget)->row_spacing);
  save_int(data, ColSpacing, GTK_TABLE(widget)->column_spacing);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_table_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_table_init()
{
  /* Initialise the GTK type */
  gtk_table_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = table_xpm;
  gbwidget.tooltip = "Table";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_table_new;
  gbwidget.gb_widget_create_properties	= gb_table_create_properties;
  gbwidget.gb_widget_show_properties	= gb_table_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_table_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_table_create_popup_menu;
  gbwidget.gb_widget_load		= gb_table_load;
*/
  gbwidget.gb_widget_save		= gb_table_save;
/*
  gbwidget.gb_widget_write_source	= gb_table_write_source;
*/

  return &gbwidget;
}

