/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/toolbar.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Orientation	= "GtkToolbar::orientation";
static gchar *Style		= "GtkToolbar::style";
static gchar *Space		= "GtkToolbar::space";
static gchar *Tooltips		= "GtkToolbar::tooltips";

static gchar* GbOrientationChoices[] = { "Horizontal", "Vertical", NULL };
static gint GbOrientationValues[] = {
  GTK_ORIENTATION_HORIZONTAL,
  GTK_ORIENTATION_VERTICAL
};

static gchar* GbStyleChoices[] = { "Icons", "Text", "Both", NULL };
static gint GbStyleValues[] = {
  GTK_TOOLBAR_ICONS,
  GTK_TOOLBAR_TEXT,
  GTK_TOOLBAR_BOTH
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkToolbar, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
/*
GtkWidget*
gb_toolbar_new(GbWidgetNewData *data)
{

}
*/



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_toolbar_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Orientation, "Orientation:",
		      "The toolbar orientation",
		      GbOrientationChoices);
  property_add_choice(Style, "Style:",
		      "The toolbar style",
		      GbStyleChoices);
  property_add_int_range(Space, "Space Size:",
			 "The size of spaces in the toolbar",
			 0, 1000, 1, 10, 1);
  property_add_bool(Tooltips, "Tooltips:", "If tooltips are enabled");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_toolbar_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbOrientationValues)
	 / sizeof(GbOrientationValues[0]); i++) {
    if (GbOrientationValues[i] == GTK_TOOLBAR(widget)->orientation)
      property_set_choice(Orientation, i);
  }

  for (i = 0; i < sizeof(GbStyleValues) / sizeof(GbStyleValues[0]); i++) {
    if (GbStyleValues[i] == GTK_TOOLBAR(widget)->style)
      property_set_choice(Style, i);
  }

  property_set_int(Space, GTK_TOOLBAR(widget)->space_size);
  property_set_bool(Tooltips,
		    GTK_TOOLTIPS(GTK_TOOLBAR(widget)->tooltips)->enabled);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_toolbar_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *orientation, *style;
  gint i, space;
  gboolean tooltips;

  orientation = property_get_choice(Orientation);
  if (Apply) {
    for (i = 0; i < sizeof(GbOrientationValues)
	   / sizeof(GbOrientationValues[0]);i++) {
      if (!strcmp(orientation, GbOrientationChoices[i])) {
	gtk_toolbar_set_orientation(GTK_TOOLBAR(widget),
				    GbOrientationValues[i]);
	break;
      }
    }
  }

  style = property_get_choice(Style);
  if (Apply) {
    for (i = 0; i < sizeof(GbStyleValues) / sizeof(GbStyleValues[0]);i++) {
      if (!strcmp(style, GbStyleChoices[i])) {
	gtk_toolbar_set_style(GTK_TOOLBAR(widget), GbStyleValues[i]);
	break;
      }
    }
  }

  space = property_get_int(Space);
  if (Apply) gtk_toolbar_set_space_size(GTK_TOOLBAR(widget), space);

  tooltips = property_get_bool(Tooltips);
  if (Apply) gtk_toolbar_set_tooltips(GTK_TOOLBAR(widget), tooltips);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkToolbar, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_toolbar_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkToolbar. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_toolbar_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkToolbar.
 * Note: as with load(), most work will already have been done.
 */
void
gb_toolbar_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbOrientationValues)
	 / sizeof(GbOrientationValues[0]); i++) {
    if (GbOrientationValues[i] == GTK_TOOLBAR(widget)->orientation)
      save_choice(data, Orientation, GbOrientationChoices[i]);
  }

  for (i = 0; i < sizeof(GbStyleValues) / sizeof(GbStyleValues[0]); i++) {
    if (GbStyleValues[i] == GTK_TOOLBAR(widget)->style)
      save_choice(data, Style, GbStyleChoices[i]);
  }

  save_int(data, Space, GTK_TOOLBAR(widget)->space_size);
  save_bool(data, Tooltips,
		    GTK_TOOLTIPS(GTK_TOOLBAR(widget)->tooltips)->enabled);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_toolbar_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_toolbar_init()
{
  /* Initialise the GTK type */
  gtk_toolbar_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = toolbar_xpm;
  gbwidget.tooltip = "Toolbar";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_create_properties	= gb_toolbar_create_properties;
  gbwidget.gb_widget_show_properties	= gb_toolbar_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_toolbar_apply_properties;
/*
  gbwidget.gb_widget_new		= gb_toolbar_new;
  gbwidget.gb_widget_create_popup_menu	= gb_toolbar_create_popup_menu;
  gbwidget.gb_widget_load		= gb_toolbar_load;
*/
  gbwidget.gb_widget_save		= gb_toolbar_save;
/*
  gbwidget.gb_widget_write_source	= gb_toolbar_write_source;
*/

  return &gbwidget;
}

