/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/tree.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Mode	= "GtkTree::selection_mode";
static gchar *ViewMode	= "GtkTree::view_mode";
static gchar *ViewLines	= "GtkTree::view_lines";

static gchar* GbModeChoices[] = { "Single", "Browse", "Multiple",
				  "Extended", NULL};
static gint GbModeValues[] = {
  GTK_SELECTION_SINGLE,
  GTK_SELECTION_BROWSE,
  GTK_SELECTION_MULTIPLE,
  GTK_SELECTION_EXTENDED
};

static gchar* GbViewModeChoices[] = { "Line", "Item", NULL};
static gint GbViewModeValues[] = {
  GTK_TREE_VIEW_LINE,
  GTK_TREE_VIEW_ITEM
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkTree, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
/*
GtkWidget*
gb_tree_new(GbWidgetNewData *data)
{

}
*/



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_tree_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Mode, "Select Mode:",
		      "The selection mode of the tree",
		      GbModeChoices);
  property_add_choice(ViewMode, "View Mode:",
		      "The view mode of the tree",
		      GbViewModeChoices);
  property_add_bool(ViewLines, "View Lines:",
		    "If lines are displayed between items");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_tree_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]); i++) {
    if (GbModeValues[i] == GTK_TREE(widget)->selection_mode)
      property_set_choice(Mode, i);
  }

  for (i = 0; i < sizeof(GbViewModeValues) / sizeof(GbViewModeValues[0]);
       i++) {
    if (GbViewModeValues[i] == GTK_TREE(widget)->view_mode)
      property_set_choice(ViewMode, i);
  }

  property_set_bool(ViewLines, GTK_TREE(widget)->view_line);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_tree_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *mode, *view_mode;
  gint i;
  gboolean view_lines;

  mode = property_get_choice(Mode);
  if (Apply) {
    for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]);i++) {
      if (!strcmp(mode, GbModeChoices[i])) {
	gtk_tree_set_selection_mode(GTK_TREE(widget), GbModeValues[i]);
	break;
      }
    }
  }

  view_mode = property_get_choice(ViewMode);
  if (Apply) {
    for (i = 0; i < sizeof(GbViewModeValues) / sizeof(GbViewModeValues[0]);
	 i++) {
      if (!strcmp(view_mode, GbViewModeChoices[i])) {
	gtk_tree_set_view_mode(GTK_TREE(widget), GbViewModeValues[i]);
	break;
      }
    }
  }

  view_lines = property_get_bool(ViewLines);
  if (Apply) gtk_tree_set_view_lines(GTK_TREE(widget), view_lines);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkTree, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_tree_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkTree. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_tree_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkTree.
 * Note: as with load(), most work will already have been done.
 */
void
gb_tree_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  for (i = 0; i < sizeof(GbModeValues) / sizeof(GbModeValues[0]); i++) {
    if (GbModeValues[i] == GTK_TREE(widget)->selection_mode)
      save_choice(data, Mode, GbModeChoices[i]);
  }

  for (i = 0; i < sizeof(GbViewModeValues) / sizeof(GbViewModeValues[0]);
       i++) {
    if (GbViewModeValues[i] == GTK_TREE(widget)->view_mode)
      save_choice(data, ViewMode, GbViewModeChoices[i]);
  }

  save_bool(data, ViewLines, GTK_TREE(widget)->view_line);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_tree_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_tree_init()
{
  /* Initialise the GTK type */
  gtk_tree_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = tree_xpm;
  gbwidget.tooltip = "Tree";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_create_properties	= gb_tree_create_properties;
  gbwidget.gb_widget_show_properties	= gb_tree_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_tree_apply_properties;
/*
  gbwidget.gb_widget_new		= gb_tree_new;
  gbwidget.gb_widget_create_popup_menu	= gb_tree_create_popup_menu;
  gbwidget.gb_widget_load		= gb_tree_load;
*/
  gbwidget.gb_widget_save		= gb_tree_save;
/*
  gbwidget.gb_widget_write_source	= gb_tree_write_source;
*/

  return &gbwidget;
}

