/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/viewport.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Shadow	= "GtkViewport::shadow";

static gchar *HValues[]	= {
  "GtkViewport::hvalue",
  "GtkViewport::hlower",
  "GtkViewport::hupper",
  "GtkViewport::hstep",
  "GtkViewport::hpage",
  "GtkViewport::hpage_size",
};

static gchar *VValues[]	= {
  "GtkViewport::vvalue",
  "GtkViewport::vlower",
  "GtkViewport::vupper",
  "GtkViewport::vstep",
  "GtkViewport::vpage",
  "GtkViewport::vpage_size",
};

static gchar* GbShadowChoices[] = { "None", "In", "Out",
				    "Etched In", "Etched Out", NULL};
static gint GbShadowValues[] = {
  GTK_SHADOW_NONE,
  GTK_SHADOW_IN,
  GTK_SHADOW_OUT,
  GTK_SHADOW_ETCHED_IN,
  GTK_SHADOW_ETCHED_OUT
};

/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkViewport, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_viewport_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_viewport_new(NULL, NULL);
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_viewport_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Shadow, "Shadow:", "The type of shadow of the viewport",
		      GbShadowChoices);
  property_add_adjustment(HValues, GB_ADJUST_H_LABELS);
  property_add_adjustment(VValues, GB_ADJUST_V_LABELS);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_viewport_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;
  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_VIEWPORT(widget)->shadow_type)
      property_set_choice(Shadow, i);
  }

  property_show_adjustment(HValues, GTK_VIEWPORT(widget)->hadjustment);
  property_show_adjustment(VValues, GTK_VIEWPORT(widget)->vadjustment);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_viewport_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gint i;
  gchar *shadow;

  shadow = property_get_choice(Shadow);
  if (Apply) {
    for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]);i++) {
      if (!strcmp(shadow, GbShadowChoices[i])) {
	gtk_viewport_set_shadow_type(GTK_VIEWPORT(widget), GbShadowValues[i]);
	break;
      }
    }
  }

  if (property_apply_adjustment(HValues, GTK_VIEWPORT(widget)->hadjustment))
    gtk_signal_emit_by_name (GTK_OBJECT (GTK_VIEWPORT(widget)->hadjustment),
			     "value_changed");
  if (property_apply_adjustment(VValues, GTK_VIEWPORT(widget)->vadjustment))
    gtk_signal_emit_by_name (GTK_OBJECT (GTK_VIEWPORT(widget)->vadjustment),
			     "value_changed");
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkViewport, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_viewport_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkViewport. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_viewport_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkViewport.
 * Note: as with load(), most work will already have been done.
 */
void
gb_viewport_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;
  for (i = 0; i < sizeof(GbShadowValues) / sizeof(GbShadowValues[0]); i++) {
    if (GbShadowValues[i] == GTK_VIEWPORT(widget)->shadow_type)
      save_choice(data, Shadow, GbShadowChoices[i]);
  }

  save_adjustment(data, HValues, GTK_VIEWPORT(widget)->hadjustment);
  save_adjustment(data, VValues, GTK_VIEWPORT(widget)->vadjustment);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_viewport_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_viewport_init()
{
  /* Initialise the GTK type */
  gtk_viewport_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = viewport_xpm;
  gbwidget.tooltip = "Viewport";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_viewport_new;
  gbwidget.gb_widget_create_properties	= gb_viewport_create_properties;
  gbwidget.gb_widget_show_properties	= gb_viewport_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_viewport_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_viewport_create_popup_menu;
  gbwidget.gb_widget_load		= gb_viewport_load;
*/
  gbwidget.gb_widget_save		= gb_viewport_save;
/*
  gbwidget.gb_widget_write_source	= gb_viewport_write_source;
*/

  return &gbwidget;
}

