/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/vpaned.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Handle	= "GtkVPaned::handle_size";
static gchar *Gutter	= "GtkVPaned::gutter_size";


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkVPaned, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_vpaned_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_vpaned_new();
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  gtk_container_add(GTK_CONTAINER (new_widget), editor_new_placeholder());
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_vpaned_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_int_range(Handle, "Handle Size:", "The size of the handle",
			 1, 1000, 1, 10, 1);
  property_add_int_range(Gutter, "Gutter Size:", "The size of the gutter",
			 1, 1000, 1, 10, 1);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_vpaned_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  property_set_int(Handle, GTK_PANED(widget)->handle_size);
  property_set_int(Gutter, GTK_PANED(widget)->gutter_size);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_vpaned_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gint handle, gutter;

  handle = property_get_int(Handle);
  if (Apply) gtk_paned_handle_size(GTK_PANED(widget), handle);

  gutter = property_get_int(Gutter);
  if (Apply) gtk_paned_gutter_size(GTK_PANED(widget), gutter);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkVPaned, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_vpaned_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkVPaned. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_vpaned_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkVPaned.
 * Note: as with load(), most work will already have been done.
 */
void
gb_vpaned_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  save_int(data, Handle, GTK_PANED(widget)->handle_size);
  save_int(data, Gutter, GTK_PANED(widget)->gutter_size);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_vpaned_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_vpaned_init()
{
  /* Initialise the GTK type */
  gtk_vpaned_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = vpaned_xpm;
  gbwidget.tooltip = "Vertical Panes";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_vpaned_new;
  gbwidget.gb_widget_create_properties	= gb_vpaned_create_properties;
  gbwidget.gb_widget_show_properties	= gb_vpaned_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_vpaned_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_vpaned_create_popup_menu;
  gbwidget.gb_widget_load		= gb_vpaned_load;
*/
  gbwidget.gb_widget_save		= gb_vpaned_save;
/*
  gbwidget.gb_widget_write_source	= gb_vpaned_write_source;
*/

  return &gbwidget;
}

