/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/vruler.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *Metric	= "GtkVRuler::metric";
static gchar *Lower	= "GtkVRuler::lower";
static gchar *Upper	= "GtkVRuler::upper";
static gchar *Pos	= "GtkVRuler::pos";
static gchar *Max	= "GtkVRuler::max";

static gchar* GbMetricChoices[] = { "Pixels", "Inches", "Centimeters", NULL };
static gint GbMetricValues[] = {
  GTK_PIXELS,
  GTK_INCHES,
  GTK_CENTIMETERS
};


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkVRuler, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_vruler_new(GbWidgetNewData *data)
{
  GtkWidget *new_widget = gtk_vruler_new();
  gtk_ruler_set_range (GTK_RULER (new_widget), 0, 10, 0, 10);
  return new_widget;
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_vruler_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_choice(Metric, "Metric:",
		      "The units of the ruler",
		      GbMetricChoices);
  property_add_float(Lower, "Lower Value:",
		     "The low value of the ruler");
  property_add_float(Upper, "Upper Value:",
		     "The low value of the ruler");
  property_add_float(Pos, "Position:",
		     "The current position on the ruler");
  property_add_float(Max, "Max:",
		     "The maximum value of the ruler");
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_vruler_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gchar *metric_name;
  gint i;

  metric_name = GTK_RULER(widget)->metric->metric_name;
  for (i = 0; i < sizeof(GbMetricValues) / sizeof(GbMetricValues[0]); i++) {
    if (!strcmp(GbMetricChoices[i], metric_name))
      property_set_choice(Metric, i);
  }

  property_set_float(Lower, GTK_RULER(widget)->lower);
  property_set_float(Upper, GTK_RULER(widget)->upper);
  property_set_float(Pos, GTK_RULER(widget)->position);
  property_set_float(Max, GTK_RULER(widget)->max_size);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_vruler_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gchar *metric;
  gint i;
  gfloat lower, upper, pos, max;
  gboolean myApply;

  metric = property_get_choice(Metric);
  if (Apply) {
    for (i = 0; i < sizeof(GbMetricValues) / sizeof(GbMetricValues[0]);i++) {
      if (!strcmp(metric, GbMetricChoices[i])) {
	gtk_ruler_set_metric(GTK_RULER(widget), GbMetricValues[i]);
	break;
      }
    }
  }

  lower = property_get_float(Lower);
  myApply = Apply;
  upper = property_get_float(Upper);
  myApply |= Apply;
  pos = property_get_float(Pos);
  myApply |= Apply;
  max = property_get_float(Max);
  myApply |= Apply;
  if (myApply) gtk_ruler_set_range(GTK_RULER(widget), lower, upper, pos, max);
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkVRuler, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_vruler_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkVRuler. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_vruler_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkVRuler.
 * Note: as with load(), most work will already have been done.
 */
void
gb_vruler_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gchar *metric_name;
  gint i;

  metric_name = GTK_RULER(widget)->metric->metric_name;
  for (i = 0; i < sizeof(GbMetricValues) / sizeof(GbMetricValues[0]); i++) {
    if (!strcmp(GbMetricChoices[i], metric_name))
      save_choice(data, Metric, GbMetricChoices[i]);
  }

  save_float(data, Lower, GTK_RULER(widget)->lower);
  save_float(data, Upper, GTK_RULER(widget)->upper);
  save_float(data, Pos, GTK_RULER(widget)->position);
  save_float(data, Max, GTK_RULER(widget)->max_size);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_vruler_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_vruler_init()
{
  /* Initialise the GTK type */
  gtk_vruler_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = vruler_xpm;
  gbwidget.tooltip = "Vertical Ruler";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_vruler_new;
  gbwidget.gb_widget_create_properties	= gb_vruler_create_properties;
  gbwidget.gb_widget_show_properties	= gb_vruler_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_vruler_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_vruler_create_popup_menu;
  gbwidget.gb_widget_load		= gb_vruler_load;
*/
  gbwidget.gb_widget_save		= gb_vruler_save;
/*
  gbwidget.gb_widget_write_source	= gb_vruler_write_source;
*/

  return &gbwidget;
}

