/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include "../gb.h"

/* Include the 21x21 icon pixmap for this widget, to be used in the palette */
#include "../graphics/vscale.xpm"

/*
 * This is the GbWidget struct for this widget (see ../gbwidget.h).
 * It is initialized in the init() function at the end of this file
 */
static GbWidget gbwidget;

static gchar *DrawValue	= "GtkVScale::draw_value";
static gchar *ValuePos	= "GtkVScale::value_pos";
static gchar *Digits	= "GtkVScale::digits";
static gchar *Policy	= "GtkVScale::policy";

static gchar *VValues[]	= {
  "GtkVScale::hvalue",
  "GtkVScale::hlower",
  "GtkVScale::hupper",
  "GtkVScale::hstep",
  "GtkVScale::hpage",
  "GtkVScale::hpage_size",
};

static gchar* GbValuePosChoices[] = { "Left", "Right", "Top", "Bottom", NULL };
static gint GbValuePosValues[] = {
  GTK_POS_LEFT,
  GTK_POS_RIGHT,
  GTK_POS_TOP,
  GTK_POS_BOTTOM
};

static gchar* GbPolicyChoices[] = { "Continuous", "Discontinuous", "Delayed",
				     NULL };
static gint GbPolicyValues[] = {
  GTK_UPDATE_CONTINUOUS,
  GTK_UPDATE_DISCONTINUOUS,
  GTK_UPDATE_DELAYED
};


/******
 * NOTE: To use these functions you need to uncomment them AND add a pointer
 * to the function in the GbWidget struct at the end of this file.
 ******/

/*
 * Creates a new GtkWidget of class GtkVScale, performing any specialized
 * initialization needed for the widget to work correctly in this environment.
 * If a dialog box is used to initialize the widget, return NULL from this
 * function, and call data->callback with your new widget when it is done.
 * If the widget needs a special destroy handler, add a signal here.
 */
GtkWidget*
gb_vscale_new(GbWidgetNewData *data)
{
  return gtk_vscale_new(NULL);
}



/*
 * Creates the components needed to edit the extra properties of this widget.
 */
void
gb_vscale_create_properties(GtkWidget *widget, GbWidgetCreatePropertiesData *data)
{
  property_add_bool(DrawValue, "Show Value:", "If the scale's value is shown");
  property_add_int_range(Digits, "Digits:", "The number of digits to show",
			 0, 13, 1, 10, 1); /* More than 13 crashes! */
  property_add_choice(ValuePos, "Value Pos:",
		      "The position of the value",
		      GbValuePosChoices);
  property_add_choice(Policy, "Policy:",
		      "The update policy of the scale",
		      GbPolicyChoices);
  property_add_adjustment(VValues, GB_ADJUST_DEFAULT_LABELS);
}



/*
 * Shows the extra properties of the widget.
 */
void
gb_vscale_show_properties(GtkWidget *widget, GbWidgetShowPropertiesData *data)
{
  gint i;

  property_set_bool(DrawValue, GTK_SCALE(widget)->draw_value);

  for (i = 0; i < sizeof(GbValuePosValues) / sizeof(GbValuePosValues[0]);
       i++) {
    if (GbValuePosValues[i] == GTK_SCALE(widget)->value_pos)
      property_set_choice(ValuePos, i);
  }
  property_set_int(Digits, GTK_RANGE(widget)->digits);

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_RANGE(widget)->policy)
      property_set_choice(Policy, i);
  }

  property_show_adjustment(VValues, GTK_RANGE(widget)->adjustment);
}



/*
 * Applies the properties which have just been edited.
 */
void
gb_vscale_apply_properties(GtkWidget *widget, GbWidgetApplyPropertiesData *data)
{
  gboolean draw_value;
  gint digits, i;
  gchar *valuepos, *policy;

  draw_value = property_get_bool(DrawValue);
  if (Apply) {
    gtk_scale_set_draw_value(GTK_SCALE(widget), draw_value);
    /* Shouldn't really need to do this */
    editor_refresh_widget(widget);
  }

  valuepos = property_get_choice(ValuePos);
  if (Apply) {
    for (i = 0; i < sizeof(GbValuePosValues) / sizeof(GbValuePosValues[0]);
	 i++) {
      if (!strcmp(valuepos, GbValuePosChoices[i])) {
	gtk_scale_set_value_pos(GTK_SCALE(widget), GbValuePosValues[i]);
	break;
      }
    }
  }

  digits = property_get_int(Digits);
  if (Apply) gtk_scale_set_digits(GTK_SCALE(widget), digits);

  policy = property_get_choice(Policy);
  if (Apply) {
    for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]);i++) {
      if (!strcmp(policy, GbPolicyChoices[i])) {
	gtk_range_set_update_policy(GTK_RANGE(widget), GbPolicyValues[i]);
	break;
      }
    }
  }

  if (property_apply_adjustment(VValues, GTK_RANGE(widget)->adjustment))
    gtk_signal_emit_by_name (GTK_OBJECT (GTK_RANGE(widget)->adjustment),
			     "value_changed");
}



/*
 * Adds menu items to a context menu which is just about to appear!
 * Add commands to aid in editing a GtkVScale, with signals pointing to
 * other functions in this file.
 */
/*
void
gb_vscale_create_popup_menu(GtkWidget *widget, GbWidgetCreatePopupMenuData *data)
{

}
*/



/*
 * Loads any attribute/values for this GtkVScale. 
 * Note: most of the widgets attributes will have already been read in by
 * the standard code.
 */
/*
void
gb_vscale_load(GtkWidget *widget, GbWidgetLoadData *data)
{

}
*/


/*
 * Saves any special attribute/value pairs for this GtkVScale.
 * Note: as with load(), most work will already have been done.
 */
void
gb_vscale_save(GtkWidget *widget, GbWidgetSaveData *data)
{
  gint i;

  save_bool(data, DrawValue, GTK_SCALE(widget)->draw_value);

  for (i = 0; i < sizeof(GbValuePosValues) / sizeof(GbValuePosValues[0]);
       i++) {
    if (GbValuePosValues[i] == GTK_SCALE(widget)->value_pos)
      save_choice(data, ValuePos, GbValuePosChoices[i]);
  }
  save_int(data, Digits, GTK_RANGE(widget)->digits);

  for (i = 0; i < sizeof(GbPolicyValues) / sizeof(GbPolicyValues[0]); i++) {
    if (GbPolicyValues[i] == GTK_RANGE(widget)->policy)
      save_choice(data, Policy, GbPolicyChoices[i]);
  }

  save_adjustment(data, VValues, GTK_RANGE(widget)->adjustment);
}



/*
 * Writes the source code needed to create this widget.
 * You have to output everything necessary to create the widget here, though
 * there are some convenience functions to help.
 */
/*
void
gb_vscale_write_source(GtkWidget *widget, GbWidgetWriteSourceData *data)
{

}
*/



/*
 * Initializes the GbWidget structure.
 * I've placed this at the end of the file so we don't have to include
 * declarations of all the functions.
 */
GbWidget*
gb_vscale_init()
{
  /* Initialise the GTK type */
  gtk_vscale_get_type();

  /* Initialize the GbWidget structure */
  gb_widget_init_struct(&gbwidget);

  /* Fill in the pixmap struct & tooltip */
  gbwidget.pixmap_struct = vscale_xpm;
  gbwidget.tooltip = "Vertical Scale";

  /* Fill in any functions that this GbWidget has */
  gbwidget.gb_widget_new		= gb_vscale_new;
  gbwidget.gb_widget_create_properties	= gb_vscale_create_properties;
  gbwidget.gb_widget_show_properties	= gb_vscale_show_properties;
  gbwidget.gb_widget_apply_properties	= gb_vscale_apply_properties;
/*
  gbwidget.gb_widget_create_popup_menu	= gb_vscale_create_popup_menu;
  gbwidget.gb_widget_load		= gb_vscale_load;
*/
  gbwidget.gb_widget_save		= gb_vscale_save;
/*
  gbwidget.gb_widget_write_source	= gb_vscale_write_source;
*/

  return &gbwidget;
}

