/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

#include "project.h"
#include "gbwidget.h"
#include "load.h"


void
load_project_file(GbWidgetLoadData *data)
{
  data->fp = fopen(data->filename, "r");
  if (data->fp == NULL) {
    data->status = GB_FILE_OPEN_ERROR;
    return;
  }

  load_next_tag(data);
  while(data->status == GB_OK) {
    /*
    if      (strcmp(data->tag, "<WIDGET>") != 0) load_window(data);
    if      (!strcmp(tag, "")) load_window(data);
    else if (!strcmp(tag, "")) load_window(data);
    else if (!strcmp(tag, "")) load_window(data);
    else if (!strcmp(tag, "")) load_window(data);
    */
    load_next_tag(data);
  }
  fclose(data->fp);
}


gchar *
load_next_line(GbWidgetLoadData *data)
{
  char *line = fgets(data->line_buffer, GB_LINE_BUFFER_SIZE, data->fp);
  data->line_number++;
  if (line == NULL) {
      if (ferror(data->fp)) data->status = GB_FILE_READ_ERROR;
      else data->status = GB_EOF;
  }
  return (gchar*)line;
}


void
load_next_tag(GbWidgetLoadData *data)
{
  char *line, *pos;

  data->tag_name = NULL;
  data->tag_value = NULL;
  for (;;) {
    line = load_next_line(data);
    if (data->status != GB_OK) return;
    if (line[0] == '<') break;
  }
  pos = strchr(line, '>');
  if (pos == NULL) data->status = GB_INVALID_LINE;
  else data->tag_value = pos + 1;
}


gint
load_parse_bool_value(GbWidgetLoadData *data)
{
  if (!strncmp(data->tag_value, "True", 4)) return TRUE;
  else if (!strncmp(data->tag_value, "False", 5)) return FALSE;
  data->status = GB_INVALID_VALUE;
  return FALSE;  
}


gint
load_parse_int_value(GbWidgetLoadData *data)
{
  int value = 0;
  int matched;
  matched = sscanf(data->tag_value, "%i", &value);
  if (matched != 1) data->status = GB_INVALID_VALUE;
  return (gint)value;
}


glong
load_parse_long_value(GbWidgetLoadData *data)
{
  long value = 0;
  int matched;
  matched = sscanf(data->tag_value, "%li", &value);
  if (matched != 1) data->status = GB_INVALID_VALUE;
  return (glong)value;
}


gfloat
load_parse_float_value(GbWidgetLoadData *data)
{
  float value = 0;
  int matched;
  matched = sscanf(data->tag_value, "%f", &value);
  if (matched != 1) data->status = GB_INVALID_VALUE;
  return value;
}


gdouble
load_parse_double_value(GbWidgetLoadData *data)
{
  double value = 0;
  int matched;
  matched = sscanf(data->tag_value, "%lf", &value);
  if (matched != 1) data->status = GB_INVALID_VALUE;
  return value;
}


