/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include <gtk/gtk.h>

#include "project.h"
#include "gbwidget.h"
#include "save.h"

static void save_component(GtkWidget *item, gpointer data);


gint
file_exists(gchar *filename)
{
  int status;
  struct stat filestat;

  status = stat(filename, &filestat);
  if (status == -1 && errno == ENOENT) return TRUE;
  return FALSE;
}


void
save_project_file(GbWidgetSaveData *data)
{

  data->fp = fopen(data->filename, "w");
  if (data->fp == NULL) {
    data->status = GB_FILE_OPEN_ERROR;
    return;
  }

  project_foreach_component(save_component, data);

  fclose(data->fp);
}

void
save_component(GtkWidget *component, gpointer data)
{
  ((GbWidgetSaveData*)data)->indent = 0;
  gb_widget_save(component, data);
}


void
save_indent(GbWidgetSaveData *data)
{
  gint i;
  for (i = 0; i < data->indent; i++) {
    fprintf(data->fp, "  ");
  }
}

void
save_tag(GbWidgetSaveData *data, gchar *tag_name)
{
  save_indent(data);
  fprintf(data->fp, "<%s>\n", tag_name);
}


void
save_string(GbWidgetSaveData *data, gchar *tag_name, gchar *tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%s\n", tag_name, tag_value);
}


void
save_text(GbWidgetSaveData *data, gchar *tag_name, gchar *tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%s</%s>\n", tag_name, tag_value, tag_name);
}


void
save_int(GbWidgetSaveData *data, gchar *tag_name, gint tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%i\n", tag_name, tag_value);
}


void
save_long(GbWidgetSaveData *data, gchar *tag_name, glong tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%li\n", tag_name, tag_value);
}


void
save_float(GbWidgetSaveData *data, gchar *tag_name, gfloat tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%f\n", tag_name, tag_value);
}


void
save_double(GbWidgetSaveData *data, gchar *tag_name, double tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%g\n", tag_name, tag_value);
}


void
save_bool(GbWidgetSaveData *data, gchar *tag_name, gint tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%s\n", tag_name, tag_value ? "True" : "False");
}


void
save_choice(GbWidgetSaveData *data, gchar *tag_name, gchar *tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%s\n", tag_name, tag_value);
}


void
save_filename(GbWidgetSaveData *data, gchar *tag_name, gchar *tag_value)
{
  save_indent(data);
  fprintf(data->fp, "<%s>%s\n", tag_name, tag_value);
}


void
save_adjustment(GbWidgetSaveData *data, gchar *Values[],
		GtkAdjustment *adjustment)
{
  save_float(data, Values[0], adjustment->value);
  save_float(data, Values[1], adjustment->lower);
  save_float(data, Values[2], adjustment->upper);
  save_float(data, Values[3], adjustment->step_increment);
  save_float(data, Values[4], adjustment->page_increment);
  save_float(data, Values[5], adjustment->page_size);
}
