/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>


/*
 * Shows a simple message box with a label and an 'OK' button.
 */
void
show_message_box(gchar *message)
{
  GtkWidget *dialog, *hbox, *label, *button;

  dialog = gtk_dialog_new();
  gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  /*  gtk_container_border_width (GTK_CONTAINER (dialog), 2);*/

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbox), 20);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  label = gtk_label_new(message);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);

  button = gtk_button_new_with_label("OK");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE,
		     TRUE, 20);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);
  gtk_widget_show(button);

  gtk_signal_connect_object (GTK_OBJECT (dialog), "destroy",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (dialog));
  gtk_signal_connect_object (GTK_OBJECT (dialog), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (dialog));
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (dialog));
  gtk_widget_show(dialog);
}


/*
 * Shows a dialog box with a message and a number of buttons.
 * Signal handlers can be supplied for any of the buttons.
 * The dialog is automatically destroyed when any of the buttons are clicked.
 *
 * default_button specifies the default button, numbered from 1..
 *
 * Example usage:

    gchar *buttons[] = { "Yes", "No", "Cancel", NULL };
    GtkSignalFunc signal_handlers[] = { NULL, NULL, NULL };
    show_dialog("Do you want to save the current project?", buttons, 3,
		signal_handlers, NULL);
 */
void
show_dialog(gchar *message,
	    gchar* buttons[],
	    gint default_button,
	    GtkSignalFunc signal_handlers[],
	    gpointer data)
{
  GtkWidget *dialog, *hbox, *label, *button;
  int i;

  dialog = gtk_dialog_new();
  gtk_window_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  /*gtk_container_border_width (GTK_CONTAINER (dialog), 2);*/

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbox), 20);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show(hbox);

  label = gtk_label_new(message);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);

  for (i = 0; buttons[i]; i++) {
    button = gtk_button_new_with_label(buttons[i]);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE,
		       TRUE, 0);
    if (i == default_button - 1) {
      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default (button);
    }
    gtk_widget_show(button);

    if (signal_handlers[i]) gtk_signal_connect(GTK_OBJECT (button), "clicked",
					       signal_handlers[i], data);
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
			       GTK_OBJECT (dialog));
  }

  gtk_signal_connect_object (GTK_OBJECT (dialog), "destroy",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (dialog));
  gtk_signal_connect_object (GTK_OBJECT (dialog), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
			     GTK_OBJECT (dialog));
  gtk_widget_show(dialog);
}
